@@ -6,28 +6,21 @@ if sys.version_info > (3, 0):
 else:
     from collections import Sequence
 from itertools import chain
-from pkg_resources import parse_version
-import warnings
 
 import numpy as np
 import numpy.linalg
 import scipy.linalg
 import sympy as sm
 import sympy.physics.mechanics as me
-from sympy.core.function import UndefinedFunction
+from sympy.core.function import UndefinedFunction, Derivative
 Cython = sm.external.import_module('Cython')
 theano = sm.external.import_module('theano')
 if theano:
     from sympy.printing.theanocode import theano_function
 
-import pydy
-from ..utils import PyDyDeprecationWarning
 from .cython_code import CythonMatrixGenerator
 
 
-warnings.simplefilter('once', PyDyDeprecationWarning)
-
-
 class ODEFunctionGenerator(object):
     """This is an abstract base class for all of the generators. A subclass
     is expected to implement the methods necessary to evaluate the arrays
@@ -272,16 +265,6 @@ r : dictionary
             ``function``, or ``dictionary``. The speed of each, from fast to
             slow, are ``array``, ``function``, ``dictionary``, None.
 
-        Notes
-        =====
-        The generated function still supports the pre-0.3.0 extra argument
-        style, i.e. args = {'constants': ..., 'specified': ...}, but only if
-        ``constants_arg_type`` and ``specifieds_arg_type`` are both set to
-        None. This functionality is deprecated and will be removed in 0.4.0,
-        so it's best to adjust your code to support the new argument types.
-        See the docstring for the generated function for more info on the
-        new style of arguments.
-
         """
 
         self.right_hand_side = right_hand_side
@@ -373,39 +356,6 @@ r : dictionary
         lst = '- ' + ('\n' + indentation + '- ').join([str(s) for s in syms])
         return indentation + lst
 
-    def _parse_old_style_extra_args(self, *args):
-        """Returns the post-0.3.0 style args if the pre-0.3.0 style args are
-        passed in. The pre-0.3.0 style args always have three args: (x, t,
-        d) where d is is a dictionary which should always at least contain
-        the key 'constants'. It may also contain a key 'specified'."""
-
-        # DEPRECATED : Remove before 0.4.0 release.
-        if parse_version(pydy.__version__) > parse_version('0.4.0'):
-            msg = ("The old style args, i.e. {'constants': , 'specified'}, "
-                    "for the generated function is no longer supported as "
-                    "of PyDy 0.4.0. Please remove this function.")
-
-        last_arg = args[-1]
-        try:
-            constants = last_arg['constants']
-        # ValueError is needed for older NumPy versions.
-        except (KeyError, IndexError, ValueError):
-            return args
-        else:
-            warnings.warn("The old style args, i.e. {'constants': , "
-                          "'specified'}, for the generated function will be "
-                          "removed in PyDy 0.4.0.",
-                          PyDyDeprecationWarning)
-
-            new_args = list(args[:-1])  # gets x and t
-
-            if self.specifieds is not None:
-                new_args.append(last_arg['specified'])
-
-            new_args.append(constants)
-
-            return tuple(new_args)
-
     def _convert_constants_dict_to_array(self, p):
         """Returns an array of numerical values from the constants
         dictionary in the correct order."""
@@ -439,7 +389,8 @@ r : dictionary
 
         for k, v in r.items():
             # TODO : Not sure if this is the best check here.
-            if isinstance(type(k), UndefinedFunction):
+            if (isinstance(type(k), UndefinedFunction) or
+                isinstance(k, Derivative)):
                 k = (k,)
             idx = [self.specifieds.index(symmy) for symmy in k]
             try:
@@ -460,10 +411,10 @@ r : dictionary
         else:
             # More efficient.
             try:
-                self._specifieds_values[:] = r(x, t)
+                self._specifieds_values = r(x, t)
             except TypeError:  # not callable.
                 # If not callable, then it should be a float or ndarray.
-                self._specifieds_values[:] = r
+                self._specifieds_values = r
 
         return x, t, self._specifieds_values, p
 
@@ -473,8 +424,6 @@ r : dictionary
         information is provided by the user on which type of args will be
         passed in."""
 
-        args = self._parse_old_style_extra_args(*args)
-
         args = self._parse_constants(*args)
 
         if self.specifieds is not None:
@@ -511,30 +460,10 @@ r : dictionary
         """Returns a function in the form expected by scipy.integrate.odeint
         that computes the derivatives of the states."""
 
-        # This god awful mess below exists because of the need to optimize
-        # the speed of the rhs evaluation. We unfortunately support way too
-        # many ways to pass in extra arguments to the generated rhs
-        # function. The default behavior is to parse the arguments passed
-        # into the rhs function which can add a lot of computational
-        # overhead. So we allow the user to specify what type the extra args
-        # should be for both the constants and the specifieds. The constants
-        # can be None, 'array', or 'dictionary'. The specifieds can be None,
-        # 'array', 'function', or 'dictionary'. Thus we have 12 permutations
-        # of this "switch".
-
         p_arg_type = self.constants_arg_type
         r_arg_type = self.specifieds_arg_type
 
-        def slice_x(x):
-            q = x[:self.num_coordinates]
-            u = x[self.num_coordinates:]
-            return q, u
-
         if p_arg_type is None and r_arg_type is None:
-
-            # This is the only rhs that will properly check for the
-            # pre-0.3.0 rhs args for backwards compatibility.
-
             def rhs(*args):
                 # args: x, t, p
                 # or
@@ -542,158 +471,48 @@ r : dictionary
 
                 args = self._parse_all_args(*args)
 
-                q, u = slice_x(args[0])
+                q = args[0][:self.num_coordinates]
+                u = args[0][self.num_coordinates:]
 
                 xdot = self._base_rhs(q, u, *args[2:])
 
                 return xdot
 
-        elif p_arg_type == 'array' and r_arg_type is None:
+            rhs.__doc__ = self._generate_rhs_docstring()
 
-            # This could be combined with:
-            # elif p_arg_type == 'array' and r_arg_type == 'array':
+            return rhs
 
-            def rhs(*args):
-                # args: x, t, p
-                # or
-                # args: x, t, r, p
+        if p_arg_type is 'dictionary':
+            p = lambda *li : self._convert_constants_dict_to_array(li[-1])
 
-                if self.specifieds is not None:
-                    args = self._parse_specifieds(*args)
+        else:
+            p = lambda *li : self._parse_constants(*li)[-1]
 
-                q, u = slice_x(args[0])
+        if r_arg_type is None:
+            if self.specifieds is not None:
+                r = lambda *li : self._parse_specifieds(*li)[-2]
 
-                return self._base_rhs(q, u, *args[2:])
+        elif r_arg_type is 'array':
+            r = lambda *li : (li)[-2]
 
-        elif p_arg_type == 'dictionary' and r_arg_type is None:
+        elif r_arg_type is 'dictionary':
+            r = lambda *li : self._convert_specifieds_dict_to_array(*li[:3])
 
-            # This could be combined with:
-            # elif p_arg_type == 'dictionary' and r_arg_type == 'array':
+        elif r_arg_type is 'function':
+            r = lambda *li: li[2](*li[2:])
 
-            def rhs(*args):
-                # args: x, t, p
-                # or
-                # args: x, t, r, p
+        def rhs(*args):
+            # args: x, t, p
+            # or
+            # args: x, t, r, p
 
-                if self.specifieds is not None:
-                    args = self._parse_specifieds(*args)
+            q = args[0][:self.num_coordinates]
+            u = args[0][self.num_coordinates:]
 
-                p = self._convert_constants_dict_to_array(args[-1])
-
-                q, u = slice_x(args[0])
-
-                xdot = self._base_rhs(q, u, *(args[2:-1] + (p,)))
-
-                return xdot
-
-        # All of the cases below must have specifieds, so the number of args
-        # is known. r_arg_type is forces to be None if self.specifieds is
-        # None.
-
-        elif p_arg_type is None and r_arg_type == 'array':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                args = self._parse_constants(*args)
-
-                q, u = slice_x(args[0])
-
-                return self._base_rhs(q, u, *args[2:])
-
-        elif p_arg_type == 'array' and r_arg_type == 'array':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                q, u = slice_x(args[0])
-
-                return self._base_rhs(q, u, *args[2:])
-
-        elif p_arg_type == 'dictionary' and r_arg_type == 'array':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                p = self._convert_constants_dict_to_array(args[-1])
-
-                q, u = slice_x(args[0])
-
-                return self._base_rhs(q, u, *(args[2:-1] + (p,)))
-
-        elif p_arg_type is None and r_arg_type == 'dictionary':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                args = self._parse_constants(*args)
-
-                q, u = slice_x(args[0])
-
-                r = self._convert_specifieds_dict_to_array(*args[:3])
-
-                return self._base_rhs(q, u, r, args[-1])
-
-        elif p_arg_type == 'array' and r_arg_type == 'dictionary':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                q, u = slice_x(args[0])
-
-                r = self._convert_specifieds_dict_to_array(*args[:3])
-
-                return self._base_rhs(q, u, r, args[-1])
-
-        elif p_arg_type == 'dictionary' and r_arg_type == 'dictionary':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                q, u = slice_x(args[0])
-
-                p = self._convert_constants_dict_to_array(args[-1])
-
-                r = self._convert_specifieds_dict_to_array(*args[:3])
-
-                return self._base_rhs(q, u, r, p)
-
-        elif p_arg_type is None and r_arg_type == 'function':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                q, u = slice_x(args[0])
-
-                args = self._parse_constants(*args)
-
-                r = args[2](*args[:2])
-
-                return self._base_rhs(q, u, r, args[-1])
-
-        elif p_arg_type == 'array' and r_arg_type == 'function':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                q, u = slice_x(args[0])
-
-                r = args[2](*args[:2])
-
-                return self._base_rhs(q, u, r, args[-1])
-
-        elif p_arg_type == 'dictionary' and r_arg_type == 'function':
-
-            def rhs(*args):
-                # args: x, t, r, p
-
-                q, u = slice_x(args[0])
-
-                p = self._convert_constants_dict_to_array(args[-1])
-
-                r = args[2](*args[:2])
-
-                return self._base_rhs(q, u, r, p)
+            if self.specifieds is None:
+                return self._base_rhs(q, u, p(*args))
+            else:
+                return self._base_rhs(q, u, r(*args), p(*args))
 
         rhs.__doc__ = self._generate_rhs_docstring()
 
@@ -770,6 +589,13 @@ class CythonODEFunctionGenerator(ODEFunctionGenerator):
 
     def __init__(self, *args, **kwargs):
 
+        self._options = {'tmp_dir': None,
+                         'prefix': 'pydy_codegen',
+                         'cse': True,
+                         'verbose': False}
+        for k, v in self._options.items():
+            self._options[k] = kwargs.pop(k, v)
+
         if Cython is None:
             raise ImportError('Cython must be installed to use this class.')
         else:
@@ -777,9 +603,12 @@ class CythonODEFunctionGenerator(ODEFunctionGenerator):
 
     __init__.__doc__ = ODEFunctionGenerator.__init__.__doc__
 
-    @staticmethod
-    def _cythonize(outputs, inputs):
-        return CythonMatrixGenerator(inputs, outputs).compile()
+    def _cythonize(self, outputs, inputs):
+        g = CythonMatrixGenerator(inputs, outputs,
+                                  prefix=self._options['prefix'],
+                                  cse=self._options['cse'])
+        return g.compile(tmp_dir=self._options['tmp_dir'],
+                         verbose=self._options['verbose'])
 
     def _set_eval_array(self, f):
 
@@ -1011,9 +840,7 @@ def generate_ode_function(*args, **kwargs):
         return g.generate()
 
 
-_divider = '\n        Notes\n        ====='
 _docstr = ODEFunctionGenerator.__init__.__doc__
-_before_notes, _after_notes = _docstr.split(_divider)
 _extra_parameters_doc = \
 """\
         generator : string or and ODEFunctionGenerator, optional
@@ -1028,6 +855,4 @@ _extra_parameters_doc = \
             A function which evaluates the derivaties of the states. See the
             function's docstring for more details after generation.
 """
-generate_ode_function.__doc__ = ('' * 4 + _before_notes +
-                                 _extra_parameters_doc + _divider +
-                                 _after_notes)
+generate_ode_function.__doc__ = ('' * 4 + _docstr + _extra_parameters_doc)
